<script setup>

import AdminAuthenticatedLayout from '@/Layouts/AdminAuthenticatedLayout.vue';
import { ref, onMounted, computed } from 'vue';
import { Head, usePage, useForm } from '@inertiajs/vue3';
import NavLink from '@/Components/NavLink.vue';

const page = usePage();
const carBodyStyles = computed(() => page.props.carBodyStyles.data || []);
const carMakes = computed(() => page.props.carMakes || []);
const carModels = computed(() => page.props.carModels || []);
const loading = computed(() => page.props.loading || false);
const perPageOptions = [10, 20, 50]; // Adjust as needed
const perPage = ref(perPageOptions[0]);
const firstpage = ref(null);
const lastpage = ref(null);
const nextpage = ref(null);
const previouspage = ref(null);
const totalPages = ref(null);
const currentPage = ref(null);
const urlPerPage = ref(page.props.per_page);
const flashMessage = ref('');

if(perPage != urlPerPage) {
    perPage.value = urlPerPage.value;
}

const form = useForm({
    name: '',
});

const sortDirections = ref({});
const sortBy = (key) => {
  // If the key has not been sorted before, default to ascending
  if (!sortDirections.value[key]) {
    sortDirections.value[key] = 'asc';
  } else {
    // If it was ascending before, now make it descending, and vice versa
    sortDirections.value[key] = sortDirections.value[key] === 'asc' ? 'desc' : 'asc';
  }

  const direction = sortDirections.value[key];

  carBodyStyles.value.sort((a, b) => {
    if (a[key] < b[key]) {
      return direction === 'asc' ? -1 : 1;
    }
    if (a[key] > b[key]) {
      return direction === 'asc' ? 1 : -1;
    }
    return 0;
  });
};

const sortedCarBodyStyles = computed(() => carBodyStyles.value);

const changePerPage = () => {
    window.location.href = getLinkUrl();
};

const getLinkUrl = () => {
  return `/admin/products/]?per_page=${perPage.value}`;
};

const getCarMake = (productCarMake) => {
    let productCarMakeArray;
    if (Array.isArray(productCarMake)) {
        productCarMakeArray = productCarMake;
    } else if (typeof productCarMake === 'string') {
        productCarMakeArray = productCarMake.split(',').map(value => parseInt(value.trim(), 10));
    } else {
        productCarMakeArray = []; // Default empty array
    }

    const carMakeArray = [];

    productCarMakeArray.forEach(makeId => {
        const carMake = carMakes.value.find(item => item.id === makeId);
        if (carMake) {
            carMakeArray.push(carMake.name);
        }
    });

    return carMakeArray.join(', ');
}

const getCarModel = (productCarModel) => {

    let productCarModelArray;
    if (Array.isArray(productCarModel)) {
        productCarModelArray = productCarModel;
    } else if (typeof productCarModel === 'string') {
        productCarModelArray = productCarModel.split(',').map(value => parseInt(value.trim(), 10));
    } else {
        productCarModelArray = []; // Default empty array
    }

    const carModelArray = [];

    productCarModelArray.forEach(modelId => {
        const carModel = carModels.value.find(item => item.id === modelId);
        if (carModel) {
            carModelArray.push(carModel.name);
        }
    });

    return carModelArray.join(', ');
}

const editCarBodyStyle = (bodyStyle) => {
  window.location.href = `/admin/product/bodystyle/edit/${bodyStyle.id}`;
};

const deleteCarBodyStyle = (bodyStyle) => {
  const carbodystyleID = bodyStyle.id

  if (confirm('Are you sure you want to delete this?')) {
    form.post(route('admin.products.delete.bodystyle',{id: carbodystyleID}), {
        onSuccess: () => {
            flashMessage.value = "Car body style deleted successfully!";
        },
        onError: (error) => {
            console.error(error);
        }
    });
  }
};

onMounted(() => {
    const query = usePage().props;
    firstpage.value = query.carBodyStyles.first_page_url;
    lastpage.value = query.carBodyStyles.last_page_url;
    nextpage.value = query.carBodyStyles.next_page_url;
    previouspage.value = query.carBodyStyles.prev_page_url;
    totalPages.value = query.carBodyStyles.last_page;
    currentPage.value = query.carBodyStyles.current_page;
});
</script>

<template>
    <Head title="All Body Styles" />

    <AdminAuthenticatedLayout>
        <template #header>
            <div class="flex">
                <h2 class="font-semibold text-xl text-gray-800 leading-tight">Body Styles
                    <NavLink :href="route('admin.products.addbodystyle')" :active="route().current('admin.products.addbodystyle')">
                        + Add Body Style
                    </NavLink>
                </h2>
            </div>
        </template>

        <div class="py-12">
            <div v-if="flashMessage" class="max-w-7xl mx-auto sm:px-6 lg:px-8 font-bold text-red-600">
                <div class="form-row mb-6">
                    <div class="col-md-6">
                        <h4>{{ flashMessage }}</h4>
                    </div>
                </div>
            </div>

            <div class="max-w-7xl mx-auto sm:px-6 lg:px-8">
                <label for="perPageSelect">Per Page </label>
                <select id="perPageSelect" class="sm:mx-3 sm:mb-6" v-model="perPage" @change="changePerPage">
                    <option v-for="option in perPageOptions" :key="option" :value="option">
                        {{ option }}
                    </option>
                </select>
                <nav class="flex" style="width:200px;display: inline-block;float:right;">
                    <ul class="pagination" style="float:right;">
                        <li class="page-item">
                            <a class="page-link" :href="firstpage" v-if="firstpage && currentPage != totalPages"><span aria-hidden="true">&laquo;</span></a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="previouspage + '&per_page='+perPage" v-if="previouspage">Prev</a>
                        </li>
                        <li class="page-item" style="width: 80px;text-align: center;">
                            <a class="page-link"> {{ currentPage }} of {{ totalPages }}</a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="nextpage + '&per_page='+perPage" v-if="nextpage">Next</a>
                        </li>
                        <li class="page-item">
                            <a class="page-link" :href="lastpage" v-if="lastpage && currentPage != totalPages"><span aria-hidden="true">&raquo;</span></a>
                        </li>
                    </ul>
                </nav>
                <div v-if="loading">Loading...</div>
                <table v-else-if="carBodyStyles && carBodyStyles.length" class="table table-striped">
                    <thead>
                        <tr>
                        <th @click="sortBy('name')">Body Style</th>
                        <th @click="sortBy('model')">Make</th>
                        <th @click="sortBy('make')">Model</th>
                        <th @click="sortBy('year')">Year</th>
                        <th>Actions</th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr v-for="bodyStyle in sortedCarBodyStyles" :key="bodyStyle.id">
                        <td>{{ bodyStyle.name }}</td>
                        <td>{{ getCarMake(bodyStyle.make) }}</td>
                        <td>{{ getCarModel(bodyStyle.model) }}</td>
                        <td>{{ bodyStyle.year }}</td>
                        <td>
                            <button class="px-3" @click="editCarBodyStyle(bodyStyle)"><i class="fa fa-pen text-green-600"></i></button>
                            <button class="mx-3" @click="deleteCarBodyStyle(bodyStyle)"><i class="fa fa-trash text-red-600"></i></button>
                        </td>
                        </tr>
                    </tbody>
                </table>
                <p v-else>No Car Body Styles found.</p>
            </div>
        </div>
    </AdminAuthenticatedLayout>
</template>
